#include <stdio.h>
#include "slog.h"

#define  MaxIdx4VarIrec   4
#define  MaxNumOfProc     4

#if 1 
#define  MaxAssocs  10
#define  MaxArgs    10
#else
#define  MaxAssocs  0
#define  MaxArgs    0
#endif

SLOG_rectype_t GetRecType( const SLOG_uint32 IdxType,
                           const SLOG_uint32 IdxAssocs,
                           const SLOG_uint32 IdxArgs )
{
    const unsigned int    UnitType       = 100;
    const unsigned int    UnitAssocs     = 10;
    const unsigned int    UnitArgs       = 1;
    /*  May want to check if UnitXXXX is in the correct range  */
    return ( IdxType * UnitType + IdxAssocs * UnitAssocs + IdxArgs * UnitArgs );
}

void main( int argc, char **argv )
{
    const char slog_name[] = "slogfile.data";

    const SLOG_rectype_t  VarIrec_MinIdx = 0;
    const SLOG_rectype_t  FixIrec_MinIdx = 1;
    const SLOG_rectype_t  FixIrec_MaxIdx = 5;
    const SLOG_rectype_t  MaxIdxType     = FixIrec_MaxIdx - FixIrec_MinIdx + 1;
    const SLOG_rectype_t  VarIrec_MaxAssocs = 2 * MaxIdx4VarIrec;
    SLOG      slog;
    SLOG_Irec irec;

    int  MaxIrec = 7;
    int ii, jj, idx_type, Nargs, ierr;

    SLOG_rectype_t   irec_rectype;
    SLOG_evttype_t   irec_evttype;
    SLOG_starttime_t irec_starttime;
    SLOG_duration_t  irec_duration;
    SLOG_time        irec_endtime;
    SLOG_bebit_t     irec_bebits[2];
    SLOG_nodeID_t    irec_node_id;
    SLOG_cpuID_t     irec_cpu_id;
    SLOG_threadID_t  irec_thread_id;
    SLOG_where_t     irec_where;

    SLOG_N_assocs_t  irec_Nassocs;
    SLOG_assoc_t     irec_assocs[ MaxAssocs ];
    SLOG_N_args_t    irec_Nargs;
    SLOG_arg_t       irec_args[ MaxArgs ];
    SLOG_uint32      irec_Nvtrargs = 2;

    /*
        Change the Number of Interval Records created in slogfile based on
        command line argument
    */
    if ( argc > 1 ) {
        printf( "argv[1] = _%s_\n", argv[1] );
        if ( isdigit( argv[1][0] ) )  MaxIrec = atoi( argv[1] );
    }

    /*  Initialization  */
    for ( jj = 0; jj < MaxAssocs; jj++ ) irec_assocs[ jj ] = jj+1;
    for ( jj = 0; jj < MaxArgs; jj++ )   irec_args[ jj ] = jj+1;
    irec_starttime = 0.0;
    irec_duration  = 0.0;

    slog = SLOG_OpenOutputStream( slog_name );

    printf( "Start appending records :\n" );

    printf( "\t""Append variable records:\n" );

/*
    for ( ii = 0; ii <= VarIrec_MaxAssocs; ii++ ) {
        irec_evttype   = ( SLOG_evttype_t ) 200;
        irec_bebits[0] = ( SLOG_bebit_t ) 0;
        irec_bebits[1] = ( SLOG_bebit_t ) 1;
        irec_starttime = ( SLOG_starttime_t ) ii*0.1;
        irec_duration  = ( SLOG_duration_t ) 0.1;
        irec_node_id   = ( SLOG_nodeID_t ) ii % MaxNumOfProc;
        irec_cpu_id    = ( SLOG_cpuID_t ) 1;
        irec_thread_id = ( SLOG_threadID_t ) ii / MaxNumOfProc;
        irec_where     = ( SLOG_where_t ) 8888;

        irec_Nassocs   = ii % ( MaxIdx4VarIrec + 1 );
        irec_Nargs     = 2;
        irec_rectype   = GetRecType( VarIrec_MinIdx, irec_Nassocs, irec_Nargs );

        irec = SLOG_Irec_Create();
        SLOG_Irec_SetMinRec( irec, irec_rectype, irec_evttype, 
                             irec_bebits[0], irec_bebits[1],
                             irec_starttime, irec_duration,
                             irec_node_id, irec_cpu_id, irec_thread_id,
                             irec_where );
        if ( irec_Nassocs > 0 )
            SLOG_Irec_SetAssocs( irec, slog, irec_Nassocs, irec_assocs );
        for ( jj = 0; jj < irec_Nvtrargs; jj++ ) {
            if ( irec_Nargs > 0 )
               SLOG_Irec_SetArgs( irec, slog, irec_Nargs, irec_args );
            irec_Nargs++;
        }

        printf( "MAIN: %d, irec = ( "
                fmt_rtype_t", "fmt_stime_t", "fmt_dura_t", "
                fmt_Nassocs_t" )\n", 
                ii, irec_rectype, irec_starttime, irec_duration,
                irec_Nassocs );

        SLOG_Irec_ToOutputStream( slog, irec );
        SLOG_Irec_Free( irec );
    }
*/

    irec_endtime = irec_starttime + irec_duration;

    printf( "\t""Append fixed records:\n" );

    for ( ii = 0; ii < MaxIrec; ii++ ) {

        irec_evttype   = ( SLOG_evttype_t ) 200;
        irec_bebits[0] = ( SLOG_bebit_t ) 0;
        irec_bebits[1] = ( SLOG_bebit_t ) 1;
        irec_starttime = ( SLOG_starttime_t ) (ii*0.1 + irec_endtime );
        irec_duration  = ( SLOG_duration_t ) 0.1;
        irec_node_id   = ( SLOG_nodeID_t ) ii % MaxNumOfProc;
        irec_cpu_id    = ( SLOG_cpuID_t ) 1;
        irec_thread_id = ( SLOG_threadID_t ) ii / MaxNumOfProc;
        irec_where     = ( SLOG_where_t ) 8888;

        /*
          - Check if the shuffle of criteria exceeding ( frame's endtime )
            interval record to next frame's incoming records section works :
          - Elongate the endtime of 2nd records ( in current frame ) to see if 
            an extra record is written in the following frameS
        */
        if ( ii == 3 ) irec_duration  = ( SLOG_duration_t ) 0.55;

        /*
          - Check the support of variable length Association arguments is OK.
          - Check if the support of variable length MPI arguments is OK.
            Modify the irec_rectype to reflect the number of
            association arguments attached in the interval records.
        */

        idx_type      = ii % MaxIdxType;

        switch ( 0 ) {
        case 0 : 
            irec_Nassocs  = 0;
            irec_Nargs    = 0;
            break;
        case 1 :
            irec_Nassocs  = 0;
            irec_Nargs    = ii % ( MaxIdx4VarIrec + 1 );
            break;
        case 2 :
            irec_Nassocs  = ii % ( MaxIdx4VarIrec + 1 );
            irec_Nargs    = 0;
            break;
        case 3 :
            irec_Nassocs  = ii % ( MaxIdx4VarIrec + 1 ); 
            irec_Nargs    = ii / ( MaxIdx4VarIrec + 1 );
            break;
        default :
            printf( "Wrong int in the switch statements\n" );
        }

        /*  Formulas used to match the rectype in Record Definition Table  */
        irec_rectype = GetRecType( idx_type, irec_Nassocs, irec_Nargs );

        irec = SLOG_Irec_Create();
        SLOG_Irec_SetMinRec( irec, irec_rectype, irec_evttype, 
                             irec_bebits[0], irec_bebits[1],
                             irec_starttime, irec_duration,
                             irec_node_id, irec_cpu_id, irec_thread_id,
                             irec_where );
        if ( irec_Nassocs > 0 )
            SLOG_Irec_SetAssocs( irec, slog, irec_Nassocs, irec_assocs );
        if ( irec_Nargs > 0 )
            SLOG_Irec_SetArgs( irec, slog, irec_Nargs, irec_args );

        printf( "MAIN: %d, irec = ( "
                fmt_rtype_t", "fmt_stime_t", "fmt_dura_t", "
                fmt_Nassocs_t", "fmt_Nargs_t" )\n", 
                ii, irec_rectype, irec_starttime, irec_duration,
                irec_Nassocs, irec_Nargs );

        SLOG_Irec_ToOutputStream( slog, irec );
        SLOG_Irec_Free( irec );

    }   /*  for ( ii = 0; ii < MaxIrec; ii++ )  */

    printf( "Finish appending records :\n" );

    SLOG_CloseOutputStream( slog );
}
